@@ -2,7 +2,7 @@
 #' @title Score which cell each point in the test dataset belongs to.
 #' @description
 #' This function scores each data point in the test dataset based on a trained hierarchical Voronoi tessellations model. 
-#' @param data Data frame. A data frame containing the test dataset. 
+#' @param dataset Data frame. A data frame which to be scored. Can have categorical columns if `analysis.plots` are required.
 #' @param hvt.results.model List. A list obtained from the trainHVT function 
 #' @param child.level Numeric. A number indicating the depth for which the heat map is to be plotted. 
 #' @param mad.threshold Numeric. A numeric value indicating the permissible Mean Absolute Deviation.
@@ -11,7 +11,6 @@
 #' @param normalize Logical. A logical value indicating if the dataset should be normalized. When set to TRUE,
 #'  the data (testing dataset) is standardized by ‘mean’ and ‘sd’ of the training dataset referred from the trainHVT(). 
 #'  When set to FALSE, the data is used as such without any changes.
-#' @param seed Numeric. Random Seed to preserve the repeatability
 #' @param distance_metric Character. The distance metric can be L1_Norm(Manhattan) or L2_Norm(Eucledian). L1_Norm is selected by default.
 #' The distance metric is used to calculate the distance between an n dimensional point and centroid.
 #'  The distance metric can be different from the one used during training.
@@ -19,6 +18,10 @@
 #' max will return the max of m values and mean will take mean of m values where
 #' each value is a distance between a point and centroid of the cell.
 #' @param yVar Character. A character or a vector representing the name of the dependent variable(s)
+#' @param analysis.plots Logical. A logical value indicating that the scored plot should be plotted or not. If TRUE, 
+#' the identifier column(character column) name should be supplied in `names.column` argument. The output will
+#' be a 2D heatmap plotly which gives info on the cell id and the observations of a cell.
+#' @param names.column Character. A character or a vector representing the name of the identifier column/character column.
 #' @returns Dataframe containing scored data, plots and summary
 #' @author Shubhra Prakash <shubhra.prakash@@mu-sigma.com>, Sangeet Moy Das <sangeet.das@@mu-sigma.com>
 #' @seealso \code{\link{trainHVT}} \cr \code{\link{plotHVT}}
@@ -44,19 +47,21 @@
 #' @export scoreHVT
 
 
-scoreHVT <- function(data,
+scoreHVT <- function(dataset,
                        hvt.results.model,
                        child.level = 1,
                        mad.threshold = 0.2,
-                       line.width = c(0.6, 0.4, 0.2),
+                       line.width = 0.6,
                        color.vec = c("navyblue", "slateblue", "lavender"),
                        normalize = TRUE,
-                       seed = 300,
                        distance_metric = "L1_Norm",
                        error_metric = "max",
-                       yVar = NULL
+                       yVar = NULL,
+                       analysis.plots = FALSE,
+                       names.column = NULL
                        ) {
-  set.seed(seed)
+  set.seed(300)
+  sum_n = NULL
   requireNamespace("dplyr")
   requireNamespace("purrr")
   requireNamespace("data.table")
@@ -64,43 +69,42 @@ scoreHVT <- function(data,
   if (!("Cell.ID" %in% colnames(hvt.results.model[[3]]$summary))) {
     hvt.results.model[[3]]$summary <- getCellId(hvt.results = hvt.results.model)
   }
- # browser()
+ 
+  if (analysis.plots == TRUE && is.null(names.column)) {
+   stop("names.column is not defined")
+  }
+  
+  
   hvt.results.model[[3]]$summary <- cbind(hvt.results.model[[3]]$summary, centroidRadius = unlist(hvt.results.model[[3]]$max_QE))
 
+  numeric_columns <- sapply(as.data.frame(dataset), is.numeric)
+  data <- dataset[, numeric_columns, drop = FALSE]
+
   summary_list <- hvt.results.model[[3]]
   train_colnames <- names(summary_list[["nodes.clust"]][[1]][[1]])
-  #data <- data.frame(data)
-  if (!all(train_colnames %in% colnames(data))) {
+
+    if (!all(train_colnames %in% colnames(data))) {
     stop("Not all training columns are part of test dataset")
   }
   
   data_structure_score  <- dim(data)
   
-  
-  
-  #common_cols <- intersect(colnames(data), train_colnames)
-  
   if (!all(is.na(summary_list$scale_summary)) && normalize == TRUE) {
     scaled_test_data <- scale(
-      #data[, common_cols],
       data[, train_colnames],
       center = summary_list$scale_summary$mean_data[train_colnames],
       scale = summary_list$scale_summary$std_data[train_colnames]
     )
   } else {
      scaled_test_data <- data[, train_colnames]
-    #scaled_test_data <- data[, common_cols]
-    
+ 
   }
 
   colnames(scaled_test_data) <- train_colnames
-  #colnames(scaled_test_data) <- common_cols
-  
-  # level <- length(summary_list$nodes.clust)
+ 
   level <- child.level
 
-  # keep_col <- names(summary_list$summary)
-  # subsetting df based on multiple dep variables
+  
   if (!is.null(yVar)) {
     yVardf <- data[, yVar]
     if (length(yVar) != 1) {
@@ -108,12 +112,10 @@ scoreHVT <- function(data,
     }
   }
    
-  # dfWithMapping <- summary_list$summary
 
 
   find_path <- function(data_vec, centroid_data) {
-   # browser()
-    # centroidDist <- which.min(sqrt(colSums((centroid_data - data_vec) ^ 2)))
+  
     if (distance_metric == "L1_Norm") {
       centroidDist <- which.min(colSums(abs(centroid_data - data_vec), na.rm = TRUE))
       Quant.Error <- (colSums(abs(centroid_data - data_vec), na.rm = TRUE))[centroidDist]
@@ -124,8 +126,7 @@ scoreHVT <- function(data,
     return(data.frame("Index" = centroidDist, "Quant.Error" = Quant.Error / length(train_colnames)))
   }
 
-  ## Get a df with Segment level, parent, child info joined with max QE value
-  # centroidRadius <- unlist(summary_list$max_QE)
+ 
   newdfMapping <- summary_list$summary
 
   innermostCells2 <- newdfMapping %>%
@@ -148,7 +149,6 @@ scoreHVT <- function(data,
 
   predict_test_data2 <-
     cbind(data.frame(scaled_test_data, "n" = 1), cent_dist_df2) %>%
-   # cbind(data.frame(scaled_test_data[, !(names(scaled_test_data) == yVar)], "n" = 1), cent_dist_df2) %>%
     dplyr::left_join(
       innermostCells2 %>%
         select(all_of(groupCols2)) %>%
@@ -251,6 +251,26 @@ scoreHVT <- function(data,
     unlist(., recursive = FALSE) %>%
     unlist(., recursive = FALSE)
 
+  
+
+  
+  
+  
+  hvt_res1 <- hvt.results.model[[2]][[1]]$`1`
+  hvt_res2 <- hvt.results.model[[3]]$summary$Cell.ID
+  a <- 1: length(hvt_res1)
+  b <- a[hvt_res2]
+  b <-  as.vector(b)
+  hvt_res2 <- stats::na.omit(b)
+  
+  coordinates_value1 <- lapply(1:length(hvt_res1), function(x) {
+    centroids1 <- hvt_res1[[x]]
+    coordinates1 <- centroids1$pt})
+  cellID_coordinates <- do.call(rbind.data.frame, coordinates_value1)
+  colnames(cellID_coordinates) <- c("x", "y")
+  cellID_coordinates$Cell.ID <- hvt_res2
+#browser()
+  ##################
   boundaryCoords2 <-
     lapply(plotList, function(x) {
       data.frame(
@@ -263,14 +283,17 @@ scoreHVT <- function(data,
         "bp.y" = I(x$y)
       )
     }) %>%
-    bind_rows(.) %>%
+    bind_rows(.) 
+  
+  
+  boundaryCoords2 <- merge(boundaryCoords2, cellID_coordinates, by = c("x" ,"y")) %>%
     right_join(.,
-      QECompareDf2 %>% dplyr::filter(anomalyFlag == 1),
-      by = paste0("Segment.", c("Level", "Parent", "Child"))
+               QECompareDf2 %>% dplyr::filter(anomalyFlag == 1),
+               by = paste0("Segment.", c("Level", "Parent", "Child"))
     )
-
-
-  predictPlot <- plotHVT(
+  
+  
+  anomalyPlot <- plotHVT(
     hvt.results.model,
     line.width = line.width,
     color.vec = color.vec,
@@ -304,14 +327,14 @@ scoreHVT <- function(data,
     ) +
     scale_x_continuous(expand = c(0, 0)) +
     scale_y_continuous(expand = c(0, 0))
-
-
+  
+  
   colour_scheme <- c(
     "#6E40AA", "#6B44B2", "#6849BA", "#644FC1", "#6054C8", "#5C5ACE", "#5761D3", "#5268D8", "#4C6EDB", "#4776DE", "#417DE0", "#3C84E1", "#368CE1",
     "#3194E0", "#2C9CDF", "#27A3DC", "#23ABD8", "#20B2D4", "#1DBACE", "#1BC1C9", "#1AC7C2", "#19CEBB", "#1AD4B3", "#1BD9AB", "#1DDFA3", "#21E39B",
     "#25E892", "#2AEB8A", "#30EF82", "#38F17B", "#40F373", "#49F56D", "#52F667", "#5DF662", "#67F75E", "#73F65A", "#7FF658", "#8BF457", "#97F357", "#A3F258"
   )
-
+  
   if (nrow(boundaryCoords2) != 0) {
     hoverText <- paste(
       " Cell ID:",
@@ -334,7 +357,7 @@ scoreHVT <- function(data,
     hoverText <- NULL
   }
   # browser()
-  predictPlot <- predictPlot + geom_polygon(
+  anomalyPlot <- anomalyPlot + geom_polygon(
     data = boundaryCoords2,
     aes(
       x = bp.x,
@@ -349,9 +372,9 @@ scoreHVT <- function(data,
     geom_point(data = boundaryCoords2 %>% distinct(x, y), aes(x = x, y = y), size = 1.5) +
     scale_fill_gradientn(colours = colour_scheme) +
     guides(colour = "none")
-
-  plotlyPredict <- plotly::ggplotly(predictPlot, tooltip = "text")
-
+  
+  plotlyPredict <- plotly::ggplotly(anomalyPlot, tooltip = "text")
+  
   hoverText <- lapply(plotlyPredict$x$data, function(x) {
     if (!is.null(x$text)) {
       return(x$text)
@@ -373,19 +396,15 @@ scoreHVT <- function(data,
     ) %>%
     plotly::style(plotlyPredict, hoverinfo = "none", traces = trace_vec) %>%
     plotly::config(displayModeBar = FALSE)
-
-  predict_test_data3 <- predict_test_data3 %>% mutate_if(is.numeric, round, digits = 4) # Rounding decimal columns using dplyr function
+  
+  
+  predict_test_data3 <- predict_test_data3 %>% mutate_if(is.numeric, round, digits = 4) 
+  predict_test_data3 <- predict_test_data3 %>% dplyr::select(c("Segment.Level",  "Segment.Parent" ,"Segment.Child" , "n" ,            
+                                                               "Cell.ID", "Quant.Error",  "centroidRadius" ,"diff" , "anomalyFlag", everything()))
   predict_test_dataRaw <- predict_test_data3
-  #common_cols <- intersect(colnames(data), train_colnames)
-  #predict_test_dataRaw <- data[, common_cols]
   predict_test_dataRaw[, train_colnames] <- data[, train_colnames]
-
-  ################################################# Changes #################
-
-
-
+################################################# Changes #################
   predicted_result <- hvt.results.model[[3]]$summary 
-  #predicted_result <- round(predicted_result,4)
   current_predicted <- colnames(predicted_result)
   new_names <- paste0("pred_", current_predicted)
   colnames(predicted_result) <- new_names
@@ -427,7 +446,6 @@ scoreHVT <- function(data,
     }
     temp0 <- temp0 %>% purrr::discard(~ all(is.na(.) | . == ""))
     df_new[, 1] <- rowMeans(temp0)
-    #df_new <- round(df_new,4)
     return(df_new)
   }
 
@@ -438,15 +456,170 @@ scoreHVT <- function(data,
   merged_result <- rename(merged_result, c("diff" = "diff"))
 
 
-  # Define the desired column order
   desired_order <- c("Row.No", grep("^act_", colnames(merged_result), value = TRUE), "Cell.ID", grep("^pred_", colnames(merged_result), value = TRUE), "diff")
-
-  # Reorder the columns in the data frame
   df_reordered <- merged_result[, desired_order]
+#################################################
 
+  
+if(analysis.plots) { 
+  
+  
+    scored_data <- data.frame(Cell.ID = predict_test_data3$Cell.ID, names.column)
+    scored_data <- scored_data[order(scored_data$Cell.ID), c("Cell.ID", "names.column")]
+    reformed_data <- stats::aggregate(names.column ~ Cell.ID, scored_data, FUN = function(x) paste(x, collapse = ", "))
+    
+    boundaryCoords2_1 <-
+      lapply(plotList, function(x) {
+        data.frame(
+          "Segment.Level" = x[["Segment.Level"]],
+          "Segment.Parent" = x[["Segment.Parent"]],
+          "Segment.Child" = x[["Segment.Child"]],
+          "x" = x$pt["x"],
+          "y" = x$pt["y"],
+          "bp.x" = I(x$x),
+          "bp.y" = I(x$y)
+        )
+      }) %>%
+      bind_rows(.) 
+    
+    boundaryCoords2_1 <- merge(boundaryCoords2_1, cellID_coordinates, by = c("x" ,"y")) %>%
+      right_join(.,
+                 QECompareDf2,
+                 by = paste0("Segment.", c("Level", "Parent", "Child"))
+      )
+    
+    boundaryCoords2_1 <- merge(boundaryCoords2_1, reformed_data, by = "Cell.ID")
+    
+    
+    boundaryCoords2_1<- boundaryCoords2_1 %>%
+      group_by(Cell.ID) %>%
+      mutate(
+        sum_n = ifelse(any(anomalyFlag == 0) & any(anomalyFlag == 1), sum(n[anomalyFlag == 0][1], n[anomalyFlag == 1][1]), NA),
+        n = ifelse(!is.na(sum_n), sum_n, n)
+      ) %>%
+      select(-sum_n)
 
-  #browser()
-  #################################################
+    
+    scoredPlot <- plotHVT(
+      hvt.results.model,
+      line.width = line.width,
+      color.vec = color.vec,
+      centroid.size = 1.5,
+      maxDepth = child.level
+    ) +
+      theme(
+        plot.title = element_text(
+          size = 18,
+          hjust = 0.5,
+          margin = margin(0, 0, 20, 0)
+        ),
+        # legend.title = element_blank(),
+        panel.grid.major = element_blank(),
+        panel.grid.minor = element_blank(),
+        panel.background = element_blank(),
+        axis.text.x = element_blank(),
+        axis.ticks.x = element_blank(),
+        axis.title.x = element_blank(),
+        axis.text.y = element_blank(),
+        axis.ticks.y = element_blank(),
+        axis.title.y = element_blank(),
+        legend.position = "bottom"
+      ) +
+      scale_x_continuous(expand = c(0, 0)) +
+      scale_y_continuous(expand = c(0, 0))
+    
+    
+    colour_scheme <- c(
+      "#6E40AA", "#6B44B2", "#6849BA", "#644FC1", "#6054C8", "#5C5ACE", "#5761D3", "#5268D8", "#4C6EDB", "#4776DE", "#417DE0", "#3C84E1", "#368CE1",
+      "#3194E0", "#2C9CDF", "#27A3DC", "#23ABD8", "#20B2D4", "#1DBACE", "#1BC1C9", "#1AC7C2", "#19CEBB", "#1AD4B3", "#1BD9AB", "#1DDFA3", "#21E39B",
+      "#25E892", "#2AEB8A", "#30EF82", "#38F17B", "#40F373", "#49F56D", "#52F667", "#5DF662", "#67F75E", "#73F65A", "#7FF658", "#8BF457", "#97F357", "#A3F258"
+    )
+    
+
+    wrap_and_limit_text <- function(text, line_length = 100, max_chars = 500) {
+      # First, limit the total text to max_chars
+      if (nchar(text) > max_chars) {
+        text <- substr(text, 1, max_chars - 3)
+        text <- paste0(text, "...")
+      }
+      
+      # Then wrap the text
+      wrapped <- sapply(seq(1, nchar(text), line_length), function(i) {
+        substr(text, i, min(i + line_length - 1, nchar(text)))
+      })
+      
+      # Join the wrapped lines with <br> for HTML line breaks
+      paste(wrapped, collapse = "<br>")
+    }
+    
+    if (nrow(boundaryCoords2_1) != 0) {
+      boundaryCoords2_1$hoverText <- apply(boundaryCoords2_1, 1, function(row) {
+        full_names <- row["names.column"]
+        formatted_names <- wrap_and_limit_text(full_names, line_length = 100, max_chars = 500)
+        
+        paste(
+          "Cell ID:", row["Cell.ID"],
+          "<br>",
+          "Number of observations:", row["n"],
+          "<br>",
+          "Name of observations:<br>", formatted_names
+        )
+      })
+    } else {
+      boundaryCoords2_1$hoverText <- NULL
+    }
+    
+    # Create the plot
+    scoredPlot <- scoredPlot +
+      geom_polygon(
+        data = boundaryCoords2_1,
+        aes(
+          x = bp.x,
+          y = bp.y,
+          group = interaction(Segment.Level, Segment.Parent, Segment.Child),
+          fill = n,
+          text = hoverText
+        ),
+        color = "black",
+        size = 0.5
+      ) +
+      scale_fill_gradientn(colours = colour_scheme) +
+      guides(colour = "none") +
+      geom_point(
+        data = boundaryCoords2_1 %>% distinct(x, .keep_all = TRUE),
+        aes(x = x, y = y, text = hoverText),
+        size = 1.5
+      )
+    
+   
+    plotlyscored <- plotly::ggplotly(scoredPlot, tooltip = "text")
+
+    plotlyscored <- plotlyscored %>%
+      plotly::layout(
+        hovermode = 'closest',
+        hoverdistance = 100,
+        hoverlabel = list(
+          bgcolor = "rgba(255,255,0,0.2)",
+          font = list(size = 10),
+          align = "left",
+          namelength = -1),
+        legend = list(
+          title = list(text = "Level"),
+          itemdoubleclick = FALSE,
+          itemclick = "toggleothers",
+          traceorder = "reversed"
+        )
+      ) %>%
+      plotly::config(displayModeBar = TRUE)
+
+#############################  
+    names_data <- boundaryCoords2_1 %>% dplyr::select("Cell.ID","names.column")    
+    names_data <- names_data %>% 
+      distinct(Cell.ID, .keep_all = TRUE)  
+    centroid_data = merge(cellID_coordinates, names_data, by = 'Cell.ID') %>% as.data.frame()
+    
+}
+ #################################################
   #MODEL INFO Rewriting
   input_dataset <- hvt.results.model[["model_info"]][["input_parameters"]][["input_dataset"]]
   n_cells <-hvt.results.model[["model_info"]][["input_parameters"]][["n_cells"]]
@@ -474,20 +647,38 @@ scoreHVT <- function(data,
     no_of_anomaly_cells = no_of_anomaly_cells
   )
   
+  
   ####################################
+  if (analysis.plots){
   prediction_list <- list(
     scoredPredictedData = predict_test_data3,
     actual_predictedTable = df_reordered,
     QECompareDf = QECompareDf2,
-    predictPlot = plotlyPredict,
+    anomalyPlot = plotlyPredict,
+    scoredPlotly = plotlyscored,
+  #  scoredPlotlyData = boundaryCoords2_1,
+    centroidData = centroid_data,
+    predictInput = c("depth" = child.level, "quant.err" = mad.threshold),
+    model_mad_plots = list(),
+    model_info = list(type = "hvt_prediction",
+                      trained_model_summary = trained_model,
+                      scored_model_summary =scored_model)
+  )}else{
+  
+  prediction_list <- list(
+    scoredPredictedData = predict_test_data3,
+    actual_predictedTable = df_reordered,
+    QECompareDf = QECompareDf2,
+    anomalyPlot = plotlyPredict,
     predictInput = c("depth" = child.level, "quant.err" = mad.threshold),
     model_mad_plots = list(),
     model_info = list(type = "hvt_prediction", 
                       trained_model_summary = trained_model,
                       scored_model_summary =scored_model)
-  )
+  )}
+  
   model_mad_plots <- NA
-  # browser()
+ 
   if (!all(is.na(hvt.results.model[[4]]))) {
     mtrain <- hvt.results.model[[4]]$mad_plot_train + ggtitle("Mean Absolute Deviation Plot: Calibration on Train Data")
   }
@@ -503,3 +694,4 @@ scoreHVT <- function(data,
   prediction_list$model_mad_plots <- model_mad_plots
   return(prediction_list)
 }
+
